import { Feed, FeedStoreState } from '@knocklabs/client';
export type Selector<T> = (state: FeedStoreState) => T;
/**
 * Create a hook factory that provides access to the TanStack Store with optional selector support.
 * This pattern allows for flexible store access with or without selectors while maintaining
 * type safety. The selector can be passed either to useCreateNotificationStore or
 * useNotificationStore, with the latter taking precedence.
 */
declare function useCreateNotificationStore(feedClient: Feed): <T = FeedStoreState>(selector?: Selector<T>) => T;
/**
 * A hook used to access content within the notification store.
 *
 * @example
 *
 * ```ts
 * const { items, metadata } = useNotificationStore(feedClient);
 * ```
 *
 * A selector can be used to access a subset of the store state.
 *
 * @example
 *
 * ```ts
 * const { items, metadata } = useNotificationStore(feedClient, (state) => ({
 *   items: state.items,
 *   metadata: state.metadata,
 * }));
 * ```
 */
declare function useNotificationStore(feedClient: Feed): FeedStoreState;
declare function useNotificationStore<T>(feedClient: Feed, selector: Selector<T>): T;
export { useCreateNotificationStore };
export default useNotificationStore;
//# sourceMappingURL=useNotificationStore.d.ts.map